@@ -1,5 +1,5 @@
 package TreePath;
-$TreePath::VERSION = '0.18';
+$TreePath::VERSION = '0.19';
 use utf8;
 use Moose;
 with 'MooseX::Object::Pluggable';
@@ -346,6 +346,10 @@ sub del {
   my @deleted;
   foreach my $node ( @nodes ) {
 
+  my $parent_key = $self->_get_key_name('parent', $node);
+
+      $self->_normalize_node($node);
+
       my $father = $node->{parent};
 
       $self->_remove_child_father($node);
@@ -376,16 +380,16 @@ sub _get_key_name {
   die "the key '$key' is unknown in _get_key_name !"
       if ! defined $values->{$key};
 
-  if ( defined $self->config ) {
-      return $self->config->{backend}->{args}->{sources_name}->{$node->{source}}->{"${key}_key"} ||
-          $values->{$key};
+  if ( defined $self->config && defined $node->{source} ) {
+      my $key_name = $self->config->{backend}->{args}->{sources_name}->{$node->{source}}->{"${key}_key"};
+      return $key_name || $values->{$key};
   }
   return $values->{$key};
   };
 
 
 # checks that the necessary keys are set
-sub _check_node {
+sub _normalize_node {
   my ($self, $node, $parent) = @_;
 
   # check parent key
@@ -399,41 +403,23 @@ sub _check_node {
       die "key $parent_key is not set in node !"
           if ! defined $node->{$parent_key} && $self->root;
   }
-}
-
-
-# Inserts a node beneath the parent
-sub add {
-  my ($self, $node, $parent) = @_;
-
-  $self->_log("Add node ". $self->_obj_key($node));
-
-  $self->_check_node($node, $parent);
   my $parent_key = $self->_get_key_name('parent', $node);
 
-  if ( $parent ) {
-      $node->{$parent_key} = $parent;
-  }
-
   # transform obj_id to object reference
   foreach my $col ( keys %$node ) {
 
       # hash = belongs_to
       if ( ref($node->{$col}) eq 'HASH') {
 
-          # if $parent and col == parent
-          if ( defined $parent && $col eq $parent_key) {
-              $node->{$col} = $parent;
-          }
-          else {
-              my @values = keys %{$node->{$col}};
+          my @values = keys %{$node->{$col}};
+          # hasref is not already transformed
+          if ( ! defined $values[1] ) {
               my $value = $values[0];
-
               if ( $value =~ m/^\w*_\d*$/ ) {
                   $node->{$col} = $self->tree->{$value};
               }
               else {
-                  die "Error: Relationship belongs_to is not well formatted : $value, use Obj_ID as form"
+                  die "Error: Relationship belongs_to is not well formatted : '$value', use Obj_ID as form"
               }
           }
       }
@@ -453,17 +439,31 @@ sub add {
           }
           $node->{$col} = $newarray;
       }
+  }
 
-      # col eq 'parent' or equivalent
-      if ( $col eq $parent_key ) {
-          $parent = $node->{$parent_key};
-          if ( ! $parent ){
-              if ( $self->root) {
-                  die "root already exist [ node: " .$node->{$self->_get_key_name('source', $node)}. '_' . $node->{$self->_get_key_name('primary', $node)} . " ]!";
-              }
-              $self->root($node);
-          }
+}
+
+
+# Inserts a node beneath the parent
+sub add {
+  my ($self, $node, $parent) = @_;
+
+  $self->_log("Add node ". $self->_obj_key($node));
+
+  my $parent_key = $self->_get_key_name('parent', $node);
+
+  if ( $parent ) {
+      $node->{$parent_key} = $parent;
+  }
+
+  $self->_normalize_node($node, $parent);
+
+  $parent = $node->{$parent_key};
+  if ( ! $parent ){
+      if ( $self->root) {
+          die "root already exist [ node: " .$node->{$self->_get_key_name('source', $node)}. '_' . $node->{$self->_get_key_name('primary', $node)} . " ]!";
       }
+      $self->root($node);
   }
 
   # save ref node in tree
@@ -487,6 +487,9 @@ sub add {
 sub update {
   my ($self, $node, $datas) = @_;
 
+  $self->_log("Update node ". $self->_obj_key($node));
+  $self->_normalize_node($node);
+
   foreach my $k ( sort keys %$datas ) {
 
       if ( ! defined $node->{$k} ){
@@ -496,23 +499,18 @@ sub update {
       elsif ( $node->{$k} eq $datas->{$k} ){next}
 
       my $previous = $node->{$k};
-      my $parent   = $node->{parent};
+      my $parent   = $node->{$self->_get_key_name('parent', $node)};
       my $key_children = $self->_key_children($node, $parent);
       my $children = $parent->{$key_children};
 
 
-      if ( $k eq 'parent') {
+      if ( $k eq $self->_get_key_name('parent', $node)) {
 
           my $old = $self->_remove_child_father($node);
           my $new_parent = $datas->{$k};
           push(@{$new_parent->{$key_children}}, $old);
 
-          $node->{parent} = $new_parent;
-
-          my $msg = 'updating parent of '. $node->{$self->_get_key_name('search',$node)} .
-              " ( " . $old->{name} . " -> ". $node->{parent}->{name} .")";
-
-          #$self->_bckd_update($node, $msg);
+          $node->{$self->_get_key_name('parent', $node)} = $new_parent;
       }
       else {
           $node->{$k} = $datas->{$k};
@@ -525,7 +523,8 @@ sub update {
 sub move {
     my ($self, $node, $parent) = @_;
 
-    return $self->update($node, { parent => $parent });
+    $self->_normalize_node($node);
+    return $self->update($node, { $self->_get_key_name('parent', $node) => $parent });
 }
 
 =head1 NAME
