@@ -13,8 +13,7 @@ use MooseX::MarkAsMethods autoclean => 1;
 extends 'DBIx::Class::Core';
 
 
-__PACKAGE__->load_components(
-    qw/DateTime::Epoch TimeStamp Core/);
+__PACKAGE__->load_components;
 __PACKAGE__->table("comments");
 __PACKAGE__->add_columns(
     "id",
